/**
* Returns the int RGBA value for the given float4
*/
uint rgbaFloatToInt(float4 rgba)
{
    rgba.x = clamp(rgba.x,0.0f,1.0f);  
    rgba.y = clamp(rgba.y,0.0f,1.0f);  
    rgba.z = clamp(rgba.z,0.0f,1.0f);  
    rgba.w = clamp(rgba.w,0.0f,1.0f);  
    return ((uint)(rgba.w*255.0f)<<24) | ((uint)(rgba.z*255.0f)<<16) | ((uint)(rgba.y*255.0f)<<8) | (uint)(rgba.x*255.0f);
} //end of rgbaFloatToInt function

/**
* Main kernel function.  Takes pArray and maxValue and copies the clamped, normalized values to pixels.
*/
__kernel void normalizeImage(int imageWidth, int imageHeight, float maxValue, __global float *input, __global uint *pixels)
{
    uint x = get_global_id(0);
    uint y = get_global_id(1);
    
    // write output color
    uint index =(y * imageWidth) + x;
    
    float v = input[index]/maxValue;
    
    //Create a flow pixel    
    float4 temp = (float4)(v,v,v,0);
    //Convert to a pixel rgb value
    pixels[index] = rgbaFloatToInt(temp);
   
};


